// Generated by CoffeeScript 1.7.1

/*

DOCMOD
 */

(function() {
  var Q, compile, compileOrRedirect, each, exports, fs, getDoc, getDocOrRedirect, jade, layout, link, linkAndLoad, load, md, opt, path, render, request, stream, url, util, yaml;

  fs = require('fs');

  stream = require('stream');

  path = require('path');

  url = require('url');

  util = require('util');

  yaml = require('js-yaml');

  request = require('request');

  each = require('each-async');

  Q = require('q');

  jade = require('jade');

  md = require('marked');

  require('obj-uber');

  opt = {
    host: 'localhost',
    src: './src/docmod',
    out: './out/docmod',
    maxDepth: 4,
    filter: function(doc, req) {
      if (doc.published != null) {
        return doc.published;
      } else {
        return true;
      }
    }
  };

  module.exports = exports = function(arg) {
    opt = arg.uber(opt);
    return function(req, res, next) {
      return compileOrRedirect(req).then(function(doc) {
        if (doc == null) {
          return next();
        } else {
          return res.send(doc);
        }
      }).fail(function(err) {
        var urlobj, _ref;
        if (err.code === 'REDIR') {
          urlobj = url.parse(req.url);
          res.redirect(urlobj.pathname + '/' + ((_ref = urlobj.search) != null ? _ref : ''));
        }
        return next(err);
      });
    };
  };

  exports.compileOrRedirect = compileOrRedirect = function(req, overridepath) {
    var i, locals, loopLayout, pathname, srcPath, urlobj;
    urlobj = url.parse(req.url);
    pathname = overridepath != null ? overridepath : urlobj.pathname;
    srcPath = path.resolve('.', path.join(opt.src, pathname));
    locals = {
      site: opt.site,
      url: pathname
    };
    i = 0;
    return getDocOrRedirect(pathname).then(function(meta) {
      return linkAndLoad(req, meta, srcPath);
    }).then(function(meta) {
      locals.uber(meta);
      if (locals.body != null) {
        locals.body = md(locals.body);
      }
      if (locals.template != null) {
        return render(locals);
      } else {
        return locals;
      }
    }).then(loopLayout = function(locals) {
      i++;
      if (i > opt.maxDepth) {
        throw new Error('Max layout depth exceeded');
      }
      return Q.when(locals).then(function(locals) {
        if (locals.layout != null) {
          return loopLayout(layout(req, locals));
        } else {
          return locals;
        }
      });
    }).then(function(locals) {
      if (!opt.filter(locals, req)) {
        return null;
      }
      if (locals.content != null) {
        return locals.content;
      } else {
        return locals;
      }
    }).fail(function(err) {
      if (err.code === 'ENOENT' || err.code === 'ENOTDIR') {
        return null;
      } else {
        throw err;
      }
    });
  };

  exports.compile = compile = function(req, overridepath) {
    var i, locals, loopLayout, pathname, srcPath, urlobj;
    console.log('compiling');
    urlobj = url.parse(req.url);
    pathname = overridepath != null ? overridepath : urlobj.pathname;
    srcPath = path.resolve('.', path.join(opt.src, pathname));
    locals = {
      site: opt.site,
      url: pathname
    };
    i = 0;
    return getDoc(pathname).then(function(result) {
      return linkAndLoad(req, result.meta, srcPath, result.isindex);
    }).then(function(meta) {
      locals.uber(meta);
      if (locals.body != null) {
        locals.body = md(locals.body);
      }
      if (locals.template != null) {
        return render(locals);
      } else {
        return locals;
      }
    }).then(loopLayout = function(locals) {
      i++;
      if (i > opt.maxDepth) {
        throw new Error('Max layout depth exceeded');
      }
      return Q.when(locals).then(function(locals) {
        if (locals.layout != null) {
          return loopLayout(layout(req, locals));
        } else {
          return locals;
        }
      });
    }).then(function(locals) {
      if (!opt.filter(locals, req)) {
        return null;
      }
      if (locals.content != null) {
        return locals.content;
      } else {
        return locals;
      }
    }).fail(function(err) {
      if (err.code === 'ENOENT' || err.code === 'ENOTDIR') {
        console.error(err.stack);
        return null;
      } else {
        throw err;
      }
    });
  };

  getDocOrRedirect = function(pathname) {
    var d, srcPath, trypath;
    d = Q.defer();
    if (pathname == null) {
      d.resolve(null);
    }
    srcPath = path.resolve('.', path.join(opt.src, pathname));
    if (pathname[pathname.length - 1] === '/') {
      trypath = path.join(srcPath, 'index.yaml');
      fs.readFile(trypath, 'utf8', function(err, data) {
        var doc;
        if (err) {
          return d.reject(err);
        } else {
          doc = yaml.safeLoad(data);
          return d.resolve(doc);
        }
      });
    } else {
      trypath = srcPath + '.yaml';
      fs.readFile(trypath, 'utf8', function(err, data) {
        var doc;
        if (err) {
          if (err.code === 'ENOENT') {
            trypath = path.join(srcPath, 'index.yaml');
            return fs.exists(trypath, function(exists) {
              if (exists) {
                err = new Error('Redirect required.');
                err.code = 'REDIR';
                return d.reject(err);
              } else {
                return d.reject(err);
              }
            });
          } else {
            return d.reject(err);
          }
        } else {
          doc = yaml.safeLoad(data);
          return d.resolve(doc);
        }
      });
    }
    return d.promise;
  };

  getDoc = function(filepath) {
    var d, trypath;
    d = Q.defer();
    if (filepath == null) {
      d.resolve(null);
    }
    trypath = filepath + '.yaml';
    fs.readFile(trypath, 'utf8', function(err, data) {
      var doc;
      if (err) {
        if (err.code === 'ENOENT') {
          trypath = path.join(filepath, 'index.yaml');
          return fs.readFile(trypath, 'utf8', function(err, data) {
            var doc;
            if (err) {
              return d.reject(err);
            } else {
              doc = yaml.safeLoad(data);
              return d.resolve({
                meta: doc,
                isindex: true
              });
            }
          });
        } else {
          return d.reject(err);
        }
      } else {
        doc = yaml.safeLoad(data);
        return d.resolve({
          meta: doc,
          isindex: false
        });
      }
    });
    return d.promise;
  };

  linkAndLoad = function(req, locals, srcPath, isindex) {
    var d, pathname;
    if (typeof locals !== 'object') {
      throw new Error('Locals (' + locals + ') is not an object');
    }
    pathname = url.parse(req.url).pathname;
    if (isindex == null) {
      isindex = pathname[pathname.length - 1] === '/';
    }
    d = Q.defer();
    each(Object.keys(locals), function(key, i, done) {
      var newProp, prop, rs;
      prop = locals[key];
      if (prop === null) {
        return done();
      }
      if (prop.hasOwnProperty('$link')) {
        locals[key] = link(req, prop.$link, srcPath, isindex);
        return done();
      } else if (prop.hasOwnProperty('$load')) {
        newProp = '';
        return rs = load(req, prop.$load, srcPath, isindex).on('data', function(data) {
          return newProp += data;
        }).on('end', function() {
          var _ref, _ref1;
          if ((_ref = rs.response) != null ? (_ref1 = _ref.headers['content-type']) != null ? _ref1.match(/application\/json/) : void 0 : void 0) {
            locals[key] = JSON.parse(newProp);
          } else {
            locals[key] = newProp.toString();
          }
          return done();
        }).on('error', function(err) {
          console.error("DocMod: Error loading '%s' at '%s'", key, req.url);
          locals[key] = '\[Error loading ' + key + '\]';
          return done();
        });
      } else if (typeof prop === 'object') {
        return linkAndLoad(req, prop, srcPath, isindex).then(function() {
          return done();
        }).fail(done);
      } else {
        return done();
      }
    }, function(err) {
      if (err) {
        return d.reject(err);
      }
      return d.resolve(locals);
    });
    return d.promise;
  };

  link = function(req, arg, srcPath, isindex) {
    var opturl, regex, reqopt, reqpath, requrl;
    reqopt = {};
    if (typeof arg === 'string') {
      reqopt.url = arg;
    } else {
      reqopt = arg;
    }
    requrl = url.parse(req.url);
    opturl = url.parse(reqopt.url);
    reqpath = reqopt.url;
    if ((opturl.hostname == null) && opturl.pathname[0] !== '/') {
      srcPath = '/' + path.relative(opt.src, srcPath);
      if (!isindex) {
        srcPath = path.resolve(srcPath, '..');
      }
      regex = /\\/g;
      reqpath = path.join(srcPath, reqopt.url).replace(regex, '/');
    }
    return reqpath;
  };

  load = function(req, arg, srcPath, isindex) {
    var opturl, pathname, reqopt, requrl;
    reqopt = {};
    if (typeof arg === 'string') {
      reqopt.url = arg;
    } else {
      reqopt = arg;
      if (reqopt.qs == null) {
        reqopt.qs = reqopt.query;
      }
    }
    requrl = url.parse(req.url);
    opturl = url.parse(reqopt.url);
    pathname = opturl.pathname;
    if (opturl.hostname == null) {
      if (opturl.pathname[0] !== '/') {
        if (srcPath != null) {
          if (!isindex) {
            srcPath = path.resolve(srcPath, '..');
          }
          pathname = path.join(srcPath, reqopt.url);
          return fs.createReadStream(pathname, {
            encoding: 'utf8'
          });
        } else {
          pathname = path.join(opturl.pathname, reqopt.url);
        }
      }
      reqopt.url = url.format({
        protocol: 'http:',
        hostname: opt.host,
        port: req.socket.localPort,
        pathname: pathname,
        search: requrl.query
      });
    }
    reqopt.headers = req.headers;
    delete reqopt.headers['accept-encoding'];
    delete reqopt.headers['content-length'];
    delete reqopt.headers['if-none-match'];
    return req.pipe(request(reqopt));
  };

  layout = function(req, locals) {
    var srcPath;
    if (locals.content == null) {
      locals.content = locals.body;
    }
    srcPath = path.resolve('.', path.join(opt.src, 'layouts', locals.layout));
    return getDoc(srcPath).then(function(result) {
      return linkAndLoad(req, result.meta, srcPath, result.isindex);
    }).then(function(meta) {
      delete locals.layout;
      locals.uber(meta);
      if (locals.template != null) {
        return render(locals);
      } else {
        return locals;
      }
    });
  };

  render = function(locals) {
    locals.basedir = path.resolve('.', path.join(opt.src, 'layouts'));
    return Q.nfcall(jade.render, locals.template, locals).then(function(rendered) {
      locals.content = rendered;
      delete locals.template;
      return locals;
    });
  };

}).call(this);
